import sys
assert sys.version_info.major == 3, 'Use ipython3 instead.'

from . import recorders
from . import detectors
from . import motors
from . import scans
from . import Gadget
from . import utils
from . import environment
